@@ -21,6 +21,7 @@ const fs = require('fs-extra');
 const path = require('path');
 const nopt = require('nopt');
 const glob = require('fast-glob');
+const dedent = require('dedent');
 const events = require('cordova-common').events;
 const AndroidManifest = require('./AndroidManifest');
 const xmlHelpers = require('cordova-common').xmlHelpers;
@@ -34,6 +35,7 @@ const utils = require('./utils');
 const gradleConfigDefaults = require('./gradle-config-defaults');
 const checkReqs = require('./check_reqs');
 const GradlePropertiesParser = require('./config/GradlePropertiesParser');
+const CordovaGradleConfigParserFactory = require('./config/CordovaGradleConfigParserFactory');
 
 function parseArguments (argv) {
     return nopt({
@@ -278,20 +280,22 @@ function updateProjectAccordingTo (platformConfig, locations) {
     // Java packages cannot support dashes
     const androidPkgName = (platformConfig.android_packageName() || platformConfig.packageName()).replace(/-/g, '_');
 
-    const manifest = new AndroidManifest(locations.manifest);
-    const manifestId = manifest.getPackageId();
+    // updating cdv-gradle-config with new androidPkgName.
+    const cdvGradleConfig = CordovaGradleConfigParserFactory.create(locations.root);
+    cdvGradleConfig.setPackageName(androidPkgName)
+        .write();
 
+    const manifest = new AndroidManifest(locations.manifest);
     manifest.getActivity()
         .setOrientation(platformConfig.getPreference('orientation'))
         .setLaunchMode(findAndroidLaunchModePreference(platformConfig));
 
     manifest.setVersionName(platformConfig.version())
         .setVersionCode(platformConfig.android_versionCode() || default_versionCode(platformConfig.version()))
-        .setPackageId(androidPkgName)
         .write();
 
     // Java file paths shouldn't be hard coded
-    const javaDirectory = path.join(locations.javaSrc, manifestId.replace(/\./g, '/'));
+    const javaDirectory = path.join(locations.javaSrc, androidPkgName.replace(/\./g, '/'));
     const java_files = glob.sync('**/*.java', { cwd: javaDirectory, absolute: true }).filter(f => {
         const contents = fs.readFileSync(f, 'utf-8');
         return /extends\s+CordovaActivity/.test(contents);
@@ -378,11 +382,12 @@ function updateProjectSplashScreen (platformConfig, locations) {
         'windowSplashScreenAnimatedIcon',
         'windowSplashScreenAnimationDuration',
         'windowSplashScreenBackground',
-        'windowSplashScreenBrandingImage',
+        'android:windowSplashScreenBrandingImage',
         'windowSplashScreenIconBackgroundColor',
         'postSplashScreenTheme'
     ].forEach(themeKey => {
-        const cdvConfigPrefKey = 'Android' + themeKey.charAt(0).toUpperCase() + themeKey.slice(1);
+        const index = themeKey.indexOf(':') + 1;
+        const cdvConfigPrefKey = 'Android' + themeKey.charAt(index).toUpperCase() + themeKey.slice(index + 1);
         const cdvConfigPrefValue = platformConfig.getPreference(cdvConfigPrefKey, this.platform);
         let themeTargetNode = splashScreenTheme.find(`item[@name="${themeKey}"]`);
 
@@ -411,7 +416,7 @@ function updateProjectSplashScreen (platformConfig, locations) {
             updateProjectSplashScreenImage(locations, themeKey, cdvConfigPrefKey, cdvConfigPrefValue);
             break;
 
-        case 'windowSplashScreenBrandingImage':
+        case 'android:windowSplashScreenBrandingImage':
             // display warning only when set.
             if (cdvConfigPrefValue) {
                 events.emit('warn', `"${themeKey}" is currently not supported by the splash screen compatibility library. https://issuetracker.google.com/issues/194301890`);
@@ -422,13 +427,14 @@ function updateProjectSplashScreen (platformConfig, locations) {
             // force the themes value to `@color/cdv_splashscreen_icon_background`
             if (!cdvConfigPrefValue && themeTargetNode) {
                 splashScreenTheme.remove(themeTargetNode);
+                delete themes.getroot().attrib['xmlns:tools'];
             } else if (cdvConfigPrefValue) {
                 // if there is no current node, create a new node.
                 if (!themeTargetNode) {
-                    themeTargetNode = themes.getroot().makeelement('item', { name: themeKey });
+                    themeTargetNode = themes.getroot().makeelement('item', { name: themeKey, 'tools:targetApi': '31' });
                     splashScreenTheme.append(themeTargetNode);
+                    themes.getroot().attrib['xmlns:tools'] = 'http://schemas.android.com/tools';
                 }
-
                 // set the user defined color.
                 themeTargetNode.text = '@drawable/ic_cdv_splashscreen_branding';
             }
@@ -537,7 +543,7 @@ function cleanupAndSetProjectSplashScreenImage (srcFile, destFilePath, possibleP
 function updateProjectSplashScreenImage (locations, themeKey, cdvConfigPrefKey, cdvConfigPrefValue = '') {
     const SPLASH_SCREEN_IMAGE_BY_THEME_KEY = {
         windowSplashScreenAnimatedIcon: 'ic_cdv_splashscreen',
-        windowSplashScreenBrandingImage: 'ic_cdv_splashscreen_branding'
+        'android:windowSplashScreenBrandingImage': 'ic_cdv_splashscreen_branding'
     };
 
     const destFileName = SPLASH_SCREEN_IMAGE_BY_THEME_KEY[themeKey] || null;
@@ -555,7 +561,7 @@ function updateProjectSplashScreenImage (locations, themeKey, cdvConfigPrefKey,
     // Default Drawable Source File
     let defaultSrcFilePath = null;
 
-    if (themeKey !== 'windowSplashScreenBrandingImage') {
+    if (themeKey !== 'android:windowSplashScreenBrandingImage') {
         try {
             // coming from user project
             defaultSrcFilePath = require.resolve('cordova-android/templates/project/res/drawable/' + destFileNameExt);
@@ -575,7 +581,7 @@ function updateProjectSplashScreenImage (locations, themeKey, cdvConfigPrefKey,
         }
 
         events.emit(emitType, emmitMessage);
-        const cleanupOnly = themeKey === 'windowSplashScreenBrandingImage';
+        const cleanupOnly = themeKey === 'android:windowSplashScreenBrandingImage';
         cleanupAndSetProjectSplashScreenImage(defaultSrcFilePath, destFilePath, possiblePreviousDestFilePath, cleanupOnly);
         return;
     }
@@ -701,8 +707,10 @@ function updateIcons (cordovaProject, platformResourcesDir) {
         mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher.png'),
         mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher_foreground.png'),
         mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher_background.png'),
+        mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher_monochrome.png'),
         mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher_foreground.xml'),
         mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher_background.xml'),
+        mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher_monochrome.xml'),
         mapImageResources(cordovaProject.root, platformResourcesDir, 'mipmap', 'ic_launcher.xml')
     );
 
@@ -726,17 +734,38 @@ function updateIconResourceForAdaptive (preparedIcons, resourceMap, platformReso
     // project's config.xml location, so we use it as base path.
     let background;
     let foreground;
+    let monochrome;
     let targetPathBackground;
     let targetPathForeground;
+    let targetPathMonochrome;
 
     for (const density in android_icons) {
         let backgroundVal = '@mipmap/ic_launcher_background';
         let foregroundVal = '@mipmap/ic_launcher_foreground';
+        const monochromeVal = '@mipmap/ic_launcher_monochrome';
 
         background = android_icons[density].background;
         foreground = android_icons[density].foreground;
+        monochrome = android_icons[density].monochrome;
 
-        if (!background || !foreground) {
+        const hasAdaptiveIcons = !!background && !!foreground;
+        let hasMonochromeIcon = !!monochrome;
+
+        if (hasMonochromeIcon && !hasAdaptiveIcons) {
+            // If we have a monochrome icon, but no adaptive icons,
+            // then warn that in order to use monochrome, the adaptive icons
+            // must be supplied. We will ignore monochrome and proceed with the
+            // icon preparation however.
+            hasMonochromeIcon = false;
+            monochrome = undefined;
+            events.emit('warn', dedent`
+                Monochrome icon found but without adaptive properties.
+                Monochrome icon requires the adaptive background and foreground assets.
+                See https://cordova.apache.org/docs/en/latest/config_ref/images.html fore more information.
+            `);
+        }
+
+        if (!hasAdaptiveIcons) {
             // This icon isn't an adaptive icon, so skip it
             continue;
         }
@@ -767,12 +796,38 @@ function updateIconResourceForAdaptive (preparedIcons, resourceMap, platformReso
             resourceMap[targetPathForeground] = android_icons[density].foreground;
         }
 
+        if (hasMonochromeIcon) {
+            if (path.extname(path.basename(monochrome)) === '.xml') {
+                // Vector Use Case
+                targetPathMonochrome = getAdaptiveImageResourcePath(platformResourcesDir, 'mipmap', density, 'ic_launcher_monochrome.xml', path.basename(android_icons[density].monochrome));
+                resourceMap[targetPathMonochrome] = android_icons[density].monochrome;
+            } else if (path.extname(path.basename(monochrome)) === '.png') {
+                // Images Use Case
+                targetPathMonochrome = getAdaptiveImageResourcePath(platformResourcesDir, 'mipmap', density, 'ic_launcher_monochrome.png', path.basename(android_icons[density].monochrome));
+                resourceMap[targetPathMonochrome] = android_icons[density].monochrome;
+            }
+        }
+
         // create an XML for DPI and set color
-        const icLauncherTemplate = `<?xml version="1.0" encoding="utf-8"?>
-<adaptive-icon xmlns:android="http://schemas.android.com/apk/res/android">
-    <background android:drawable="` + backgroundVal + `" />
-    <foreground android:drawable="` + foregroundVal + `" />
-</adaptive-icon>`;
+        let icLauncherTemplate = '';
+        if (hasMonochromeIcon) {
+            icLauncherTemplate = dedent`
+                <?xml version="1.0" encoding="utf-8"?>
+                <adaptive-icon xmlns:android="http://schemas.android.com/apk/res/android">
+                    <background android:drawable="${backgroundVal}" />
+                    <foreground android:drawable="${foregroundVal}" />
+                    <monochrome android:drawable="${monochromeVal}" />
+                </adaptive-icon>
+            `;
+        } else {
+            icLauncherTemplate = dedent`
+                <?xml version="1.0" encoding="utf-8"?>
+                <adaptive-icon xmlns:android="http://schemas.android.com/apk/res/android">
+                    <background android:drawable="${backgroundVal}" />
+                    <foreground android:drawable="${foregroundVal}" />
+                </adaptive-icon>
+            `;
+        }
 
         const launcherXmlPath = path.join(platformResourcesDir, 'mipmap-' + density + '-v26', 'ic_launcher.xml');
 
@@ -786,6 +841,7 @@ function updateIconResourceForAdaptive (preparedIcons, resourceMap, platformReso
     if (default_icon && !android_icons.mdpi) {
         let defaultTargetPathBackground;
         let defaultTargetPathForeground;
+        let defaultTargetPathMonochrome;
 
         if (background.startsWith('@color')) {
             // Colors Use Case
@@ -812,6 +868,18 @@ function updateIconResourceForAdaptive (preparedIcons, resourceMap, platformReso
             defaultTargetPathForeground = getAdaptiveImageResourcePath(platformResourcesDir, 'mipmap', 'mdpi', 'ic_launcher_foreground.png', path.basename(default_icon.foreground));
             resourceMap[defaultTargetPathForeground] = default_icon.foreground;
         }
+
+        if (monochrome) {
+            if (path.extname(path.basename(monochrome)) === '.xml') {
+                // Vector Use Case
+                defaultTargetPathMonochrome = getAdaptiveImageResourcePath(platformResourcesDir, 'mipmap', 'mdpi', 'ic_launcher_monochrome.xml', path.basename(default_icon.monochrome));
+                resourceMap[defaultTargetPathMonochrome] = default_icon.monochrome;
+            } else if (path.extname(path.basename(monochrome)) === '.png') {
+                // Images Use Case
+                defaultTargetPathMonochrome = getAdaptiveImageResourcePath(platformResourcesDir, 'mipmap', 'mdpi', 'ic_launcher_monochrome.png', path.basename(default_icon.monochrome));
+                resourceMap[defaultTargetPathMonochrome] = default_icon.monochrome;
+            }
+        }
     }
 
     return resourceMap;
@@ -882,6 +950,11 @@ function prepareIcons (icons) {
                 const favor = {};
 
                 // populating found icon.
+                if (icon.background && icon.foreground && icon.monochrome) {
+                    found.background = icon.background;
+                    found.foreground = icon.foreground;
+                    found.monochrome = icon.monochrome;
+                }
                 if (icon.background && icon.foreground) {
                     found.background = icon.background;
                     found.foreground = icon.foreground;
@@ -890,6 +963,11 @@ function prepareIcons (icons) {
                     found.src = icon.src;
                 }
 
+                if (default_icon.background && default_icon.foreground && default_icon.monochrome) {
+                    favor.background = default_icon.background;
+                    favor.foreground = default_icon.foreground;
+                    favor.monochrome = default_icon.monochrome;
+                }
                 if (default_icon.background && default_icon.foreground) {
                     favor.background = default_icon.background;
                     favor.foreground = default_icon.foreground;
@@ -908,8 +986,8 @@ function prepareIcons (icons) {
     }
 
     return {
-        android_icons: android_icons,
-        default_icon: default_icon
+        android_icons,
+        default_icon
     };
 }
 
@@ -927,8 +1005,10 @@ function cleanIcons (projectRoot, projectConfig, platformResourcesDir) {
         mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher.png'),
         mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher_foreground.png'),
         mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher_background.png'),
+        mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher_monochrome.png'),
         mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher_foreground.xml'),
         mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher_background.xml'),
+        mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher_monochrome.xml'),
         mapImageResources(projectRoot, platformResourcesDir, 'mipmap', 'ic_launcher.xml')
     );
 
